<?php

declare(strict_types=1);

namespace mirzaev\parser_from_rossko;

use avadim\FastExcelWriter\Excel as excel;

use ZipArchive as zip,
	DirectoryIterator as directory;

use SQLite3 as db;

use GuzzleHttp\Client as client,
	GuzzleHttp\Psr7\Request as request,
	GuzzleHttp\Psr7\Utils as utils;

ini_set('error_reporting', E_ALL);
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);

define('VIEWS', realpath(__DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'views'));
define('STORAGE', realpath(__DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'storage'));
define('INDEX', __DIR__);

// Автозагрузка
require INDEX . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

// Инициализация инстанции абстрактной базы данных
$db = new db(STORAGE . DIRECTORY_SEPARATOR . 'products.db');

// Инициализация товаров
$products = $db->query('SELECT * FROM product;');

// Инициализация директории с документом avito
define('AVITO', STORAGE . DIRECTORY_SEPARATOR . 'avito');

// Инициализация хранилища фотографий товаров
define('IMAGES', STORAGE . DIRECTORY_SEPARATOR . 'images');

// Инициализация таблицы
$spreadsheet = excel::create(['export']);

// Инициализация рабочего листа
$worksheet = $spreadsheet->sheet();

// Инициализация заголовков
$worksheet->writeHeader([
	'id',
	'ListingFee',
	'Category',
	'GoodsType',
	'ProductType',
	'SparePartType',
	'Title',
	'Description',
	'Price',
	'ImageUrls',
	'Condition',
	'Originality',
	'Address',
	'Availability',
	'Brand',
	'OEM',
	'EMail',
	'ContactPhone',
	'AdType',
	'CompanyName',
]);

while ($product = $products->fetchArray()) {
	// Перебор товаров

	// Пропустить товары с отсутствием названия
	if (empty($product['description'])) continue;

	// Инициализация списка изображений товара
	$images = [];

	// Поиск изображений
	if (file_exists($file = IMAGES . DIRECTORY_SEPARATOR . $product['nomenclature']))
		foreach (new directory($file) as $file)
			if (strtolower($file->getExtension()) === 'jpg')
				$images[] = "https://vinauto.store/{$product['nomenclature']}/{$file->getFilename()}";

	// Около 40% товаров из Rossko без изображений и drom их не принимает! Временное решение
	if (empty($images)) continue;

	// Преобразование типов
	$product['cost'] = (float) $product['cost'];

	// Инициализация списков
	$brands = $models = $engines = [];


	foreach (json_decode((string) $product['applicability'], true) ?? [] as $brand) {
		// Перебор марок автомобилей

		// Запись в список марок
		$brands[] = $brand['name'];

		foreach ($brand['carModels'] as $model) {
			// Перебор моделей автомобилей

			// Запись в список моделей
			$models[] = $model['name'];

			foreach ($model['engines'] as $engine) {
				// Перебор двигателей автомобилей

				// Запись в список двигателей
				$engines[] = $engine;
			}
		}
	}

	// Ограничение по максимальному количеству
	$brands = array_slice($brands, 0, 5);
	$models = array_slice($models, 0, 10);
	$engines = array_slice($engines, 0, 30);

	// Форматирование названия
	$product['description'] = trim(preg_replace('/\|.*/', '', $product['description']), ' ');

	// Инициализация категорий Avito связанных с ключевыми словами
	$categories = [
		'Автосвет' => ['фара', 'свет'],
		'Аккумуляторы' => ['аккумулятор', 'батарея', 'батарейки', 'зарядное'],
		'Двигатель' => ['двигатель', 'патрубок', 'подушка', 'гидрокомпенсатор', 'датчик', 'детонации', 'заслонка', 'клапан', 'колпачок', 'кольца', 'порншневых', 'поршень', 'лямбда-зонд', 'лямбда', 'зонд', 'маховик', 'гбц', 'распылитель', 'расходомер', 'поликлиновой', 'натяжной', 'ролик', 'сальник', 'распред', 'распредвал', 'стакан', 'термостат', 'толкатель', 'фильтр', 'форсунка', 'шатун', 'шестерня', 'шкиф', 'цилиндр'],
		'Кузов' => ['кузов', 'ролик', 'рычаг', 'дверь', 'стеклоочиститель', 'трос', 'капот'],
		'Подвеска' => ['рычаг', 'стойка', 'сайлентблок', 'ремкомплект', 'опора', 'втулка', 'амортизатор', 'крестовина', 'опора', 'пневмобаллон', 'подушка', 'пружина', 'пыльник'],
		'Рулевое управление' => ['наконечник', 'тяга', 'кулак', 'подшипник', 'ступица', 'полуось', 'пыльник', 'шрус', 'шруса', 'рулевой', 'рулевая', 'рейка', 'сошка', 'ступица', 'гидроусилитель', 'шланг', 'шлейф', 'насос'],
		'Салон' => ['салон'],
		'Стёкла' => ['стекло', 'стёкла'],
		'Топливная и выхлопная системы' => ['выхлопная', 'выхлопной', 'топливный', 'топливо', 'топливная', 'виброкомпенсатор', 'пламягаситель', 'выхлоп'],
		'Тормозная система' => ['тормоз', 'тормозной', 'барабан', 'тормозные', 'abs', 'диск', 'трос', 'колодка', 'колодки', 'ремкомплект', 'шланг'],
		'Трансмиссия и привод' => ['привода', 'трансмиссии', 'диск', 'сцеп', 'сцепления', 'акпп'],
		'Электрооборудование' => ['электро', 'электрический', 'бендикс', 'бензонасос', 'втягивающее', 'генератор', 'катушка', 'диодный', 'диод', 'реле', 'рычаг', 'свеча', 'зажигания', 'стартер', 'электровентилятор', 'вентилятор'],
		'Система охлаждения' => ['охлаждение', 'бак', 'бачок', 'водяной', 'радиатор', 'термостат'],
		'Автомобиль на запчасти' => ['запчасти']
	];

	// Инициализация списка слов из названия
	$words = explode(' ', $product['description']);

	// Инициализация реестра найденных категорий Avito по совпадению слов из названия с ключевыми словами категорий Avito
	$sorted = [];

	foreach ($words as $word) {
		// Перебор слов в названии

		foreach ($categories as $avito => $keywords) {
			// Перебор категорий Avito

			// Инициализация буфера кратчайшего расстояния между словом из названия и ключевым словом категории Avito
			$shortest = null;

			// Объявление буфера ближайшего слова из названия к ключевому слову категории Avito
			$closest = null;

			foreach ($keywords as $keyword) {
				// Перебор ключевых слов категории Avito

				// Инициализация расстояния по технологии Левенштейна между словом из названия и ключевым словом категории Avito
				$levenshtein = levenshtein($word, $keyword);

				if ($levenshtein === 0) {
					// Полное совпадение слова из названия с ключевым словом

					// Запись крайтчайшего расстояния между ключевым словом и словом из названия
					$shortest = 0;

					// Запись ближайшей категории Avito к слову из названия
					$closest  = $avito;

					// Переход к следующей итерации (успех)
					continue;
				}

				if ($levenshtein <= 2 && ($levenshtein <= $shortest || $shortest === null)) {
					// Расстояние по технологии Левенштейна менее или равно 2
					// И Расстояние по технологии Левенштейна между словом из названия и ключевым словом категории Avito меньше чем в предыдущей итерации
					// ИЛИ расстояние между словом из названия и ключевым словом категории Avito ещё не инициализировано (первая итерация)

					// Запись крайтчайшего расстояния между ключевым словом и словом из названия
					$shortest = $levenshtein;

					// Запись ближайшей категории Avito к слову из названия
					$closest  = $avito;
				}
			}

			if (!empty($closest)) {
				// Вычислено вхождение в реестр подобранных категорий Avito по ключевым словам относительно слова из названия

				// Запись в реестр подобранных категорий Avito по ключевым словам относительно слова из названия
				$sorted[$closest] ??= [];
				$sorted[$closest][] = $word;
			}
		}
	}

	// Инициализация буфера максимального количества вхождений в категорию Avito по ключевым словам относительно всех слов из названия
	$maximum = null;

	// Объявление буфера категории с максимальным количеством вхождений в реестре подобранных категорий Avito по ключевым словам относительно всех слов из названия
	$category = null;

	foreach ($sorted as $avito => $entries) {
		// Перебор реестра подобранных категорий Avito по ключевым словам относительно всех слов из названия

		// Инициализация количества вхождений в категорию Avito по ключевым словам относительно всех слов из названия
		$amount = count($entries);

		if ($maximum <= $amount || $maximum === null) {
			// Число вхождений в категорию Avito по ключевым словам относительно всех слов из названия больше чем в предыдущей итерации
			// ИЛИ число вхождений в категорию Avito по ключевым словам относительно всех слов из названия ещё не инициализировано

			// Запись максимального значения вхождений в категории Avito по ключевым словам относительно всех слов из названия
			$maximum = $amount;

			// Запись категории с максимальным количеством вхождений в реестре подобранных категорий Avito по ключевым словам относительно всех слов из названия
			$category = $avito;
		}
	}

	// Запись строки
	$worksheet->writeRow([
		$product['code'],
		'Package',
		'Запчасти и аксессуары',
		'Запчасти',
		'Для автомобилей',
		$category,
		$product['description'],
		<<<TXT
Компании VIN AUTO
Специализируемся на всех видах автозапчастей, как для легковых, так и грузовых автомобилей. 
Очень быстро ответим и привезем нужную деталь 
Уточняйте цену и наличие у менеджера. 
Пункт выдачи находится в г.Красноярске по адресу Дудинская 2Б 
Часы работы: 10:00-19:00 
Отправка запчастей и деталей по всей России.

Не является публичной офертой
TXT,
		(int) ($product['cost_rossko'] + ($product['cost_rossko'] * 0.21)),
		implode('|', $images),
		'Новое',
		'Оригинал',
		'Красноярск, Дудинская улица, 2Б',
		'В наличии',
		$product['brand'],
		$product['code'],
		'vinauto555@gmail.com',
		'79950760655',
		'Товар от производителя',
		'VIN-auto -АВТОЗАПЧАСТИ',
	]);
}

// Сохранение файла
$spreadsheet->save(AVITO . DIRECTORY_SEPARATOR . 'export.xlsx');
